import { Loop } from "redux-loop";
export declare type DataStateSelector<TState, TFilterMap extends object, TDomain extends object> = (globalState: TState) => DataFilterMapState<TFilterMap, TDomain>;
export interface DataSpecs<TInput> {
    input: Partial<TInput>;
    sortBy: string | null;
    sortByDescending: boolean;
    pageSize?: number;
    pageIndex: number;
}
export declare type EntityGraph<TDomain, TMetadata = {}> = {
    [K in keyof TDomain]?: {
        [id: string]: TDomain[K] & {
            __metadata?: TMetadata;
        };
    };
};
export interface DataFilterResponse<TDomain> {
    total: number;
    matchKeys: string[];
    rootEntity?: keyof TDomain;
    entities: EntityGraph<TDomain>;
}
export interface EntityRef<TDomain, TK extends keyof TDomain> {
    __type: TK;
    key: string;
}
export interface DataFilterState<TInput> {
    lastEntered: DataSpecs<TInput> | null;
    lastApplied: DataSpecs<TInput> | null;
    lastRequest: DataSpecs<TInput> | null;
    isLoading: boolean;
    total: number;
    matchKeys: string[];
    error: any;
}
export declare type DataFilterContainerActions<TInput> = {
    setPageIndex: (index: number) => void;
    setPageSize: (size?: number) => void;
    refresh: () => void;
    setInput: (changes: Partial<TInput>) => void;
    setSortBy: (path: string, descending?: boolean) => void;
};
export interface DataFilterApiActions<TInput, TDomain> extends DataFilterContainerActions<TInput> {
    requestSuccess: (rs: DataFilterResponse<TDomain>) => void;
    requestFailure: (err: any) => void;
}
export declare type SearchApi<TInput, TDomain> = (rq: DataSpecs<TInput>) => Promise<DataFilterResponse<TDomain>>;
declare type PageIndexChangedAction<TFilterMap> = {
    type: "FILTER/PAGE_INDEX_CHANGED";
    filterName: keyof TFilterMap;
    index: number;
};
declare type PageSizeChangedAction<TFilterMap> = {
    type: "FILTER/PAGE_SIZE_CHANGED";
    filterName: keyof TFilterMap;
    size?: number;
};
declare type RefreshAction<TFilterMap> = {
    type: "FILTER/REFRESH";
    filterName: keyof TFilterMap;
};
declare type FilterInputChangedAction<TFilterMap, TK extends keyof TFilterMap = keyof TFilterMap> = {
    type: "FILTER/INPUT_CHANGED";
    filterName: TK;
    changes: Partial<TFilterMap[TK]>;
};
declare type SortByChangedAction<TFilterMap> = {
    type: "FILTER/SORT_BY_CHANGED";
    filterName: keyof TFilterMap;
    path: string;
    descending?: boolean;
};
export declare type RequestSuccessAction<TFilterMap, TDomain, TK extends keyof TFilterMap = keyof TFilterMap> = {
    type: "FILTER/REQUEST_SUCCESS";
    filterName: TK;
    request: DataSpecs<TFilterMap[TK]>;
    payload: DataFilterResponse<TDomain>;
};
export declare type ImportEntitiesAction<TDomain> = {
    type: "FILTER/IMPORT_ENTITIES";
    entities: EntityGraph<TDomain>;
};
export declare type RequestErrorAction<TFilterMap, TK extends keyof TFilterMap = keyof TFilterMap> = {
    type: "FILTER/REQUEST_ERROR";
    filterName: TK;
    request: DataSpecs<TFilterMap[TK]>;
    payload: any;
};
export declare type DataFilterMountAction<TFilterMap, TDomain, TK extends keyof TFilterMap = keyof TFilterMap> = {
    type: "FILTER/MOUNT";
    filterName: TK;
    options: Partial<DataFilterOptions<TFilterMap[TK], TDomain>>;
};
export declare type DataFilterUnmountAction<TFilterMap, TK extends keyof TFilterMap = keyof TFilterMap> = {
    type: "FILTER/UNMOUNT";
    filterName: TK;
};
export declare type DataFilterAction<TFilterMap, TDomain> = PageIndexChangedAction<TFilterMap> | PageSizeChangedAction<TFilterMap> | RefreshAction<TFilterMap> | FilterInputChangedAction<TFilterMap> | SortByChangedAction<TFilterMap> | RequestSuccessAction<TFilterMap, TDomain> | RequestErrorAction<TFilterMap> | DataFilterMountAction<TFilterMap, TDomain> | DataFilterUnmountAction<TFilterMap> | ImportEntitiesAction<TDomain>;
export declare type DataFilterOptions<TInput, TDomain> = {
    applyInitialQuery?: boolean;
    searchApi: SearchApi<TInput, TDomain> | undefined;
    initialQuery: Partial<DataSpecs<TInput>> | undefined;
};
export declare type Filters<TFilterMap extends object> = {
    [N in keyof TFilterMap]?: DataFilterState<TFilterMap[N]>;
};
export declare type Options<TFilterMap, TDomain> = {
    [N in keyof TFilterMap]?: DataFilterOptions<TFilterMap[N], TDomain>;
};
export declare type EntityCache<TFilterMap, TDomain> = EntityGraph<TDomain, {
    [referencedBy in keyof TFilterMap]?: boolean;
}>;
export declare type DataFilterMapState<TFilterMap extends object, TDomain extends object> = {
    entityCache: EntityCache<TFilterMap, TDomain>;
    mounted: Filters<TFilterMap>;
    options: Options<TFilterMap, TDomain>;
};
export declare type DataFilterMapReducer<TFilterMap extends object, TDomain extends object> = (state: DataFilterMapState<TFilterMap, TDomain> | undefined, action: DataFilterAction<TFilterMap, TDomain>) => DataFilterMapState<TFilterMap, TDomain> | Loop<DataFilterMapState<TFilterMap, TDomain>, DataFilterAction<TFilterMap, TDomain>>;
export declare type DataFilterMapOptions<TFilterMap, TDomain> = {
    apiMap: {
        [N in keyof TFilterMap]?: SearchApi<TFilterMap[N], TDomain>;
    };
};
export {};
